import datetime
import json
import os
import re
import time
import logging
from retry import retry
from selenium.common.exceptions import TimeoutException, InvalidArgumentException
from selenium.webdriver import ActionChains
from selenium.webdriver.remote.webdriver import WebDriver
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.by import By


def date_to_str(date=None, format_str="%Y-%m-%d %H:%M:%S"):
    """时期格式化成字符
        :param date: 时间
        :param format_str: %Y-%m-%d %H:%M:%S
    """
    if date is None:
        date = datetime.datetime.now()
    return date.strftime(format_str)


class BaseDriver(WebDriver):
    def __exit__(self, exc_type, exc_val, exc_tb):
        self.pwd = os.path.dirname(__file__)
        self.quit()

    def open(self, tab_index=0, url=None, timeout=0):
        try:
            handles = self.window_handles  # 获取当前窗口句柄集合（列表类型）
            if len(handles) < tab_index + 1:
                self.execute_script('window.open("","_blank");')
                while True:
                    time.sleep(0.1)
                    handles = self.window_handles  # 获取当前窗
                    if len(handles) > tab_index:
                        break
            if self.current_window_handle != handles[tab_index]:
                self.switch_to.window(handles[tab_index])
        except Exception:
            pass
        if url is not None:
            logging.info(f"chrome open {url}")
            try:
                if timeout > 0:
                    self.set_page_load_timeout(timeout)
                    self.set_script_timeout(timeout)
                self.get(url)
            except InvalidArgumentException:
                pass

    def get_timeout(self, url=None, timeout=10):
        if url is not None:
            logging.info(f"chrome open {url}")
            try:
                if timeout > 0:
                    self.set_page_load_timeout(timeout)
                    self.set_script_timeout(timeout)
                self.get(url)
            except Exception:
                pass

    def close_tab(self, tab_index):
        handles = self.window_handles  # 获取当前窗口句柄集合（列表类型）
        if len(handles) < tab_index + 1:
            return

        self.switch_to.window(handles[tab_index])
        self.close()

    def get_headers(self):
        try:
            headers = {}
            cookie = [item["name"] + "=" + item["value"] for item in self.get_cookies()]
            cookiestr = ';'.join(item for item in cookie)
            headers["cookie"] = cookiestr
            # token = re.search(r'"token": ".*?"', str(info))[0][10:-1]
            # s.headers["token"] = token
            # s.headers["Authorization"] = token
            return headers
        except:
            return None

    def scroll_end_page(self):
        elements = self.find_elements(by=By.XPATH, value="//*")

        for element in elements:
            try:
                self.execute_script("arguments[0].scrollIntoView();", element)
            except:
                pass
            # self.execute_script("document.body.scrollTop=document.body.scrollHeight")

    def switch_tab(self, tab_index):
        if len(self.window_handles) < tab_index + 1:
            return
        self.switch_to.window(self.window_handles[tab_index])

    @retry(Exception, tries=10, delay=2)
    def element_click(self, element):
        self.execute_script("arguments[0].click();", element)

    @retry(Exception, tries=10, delay=2)
    def element_action_click(self, element):
        ActionChains(self).move_to_element(element).click(element).perform()

    def wait_id_element(self, id_, timeout=10, displayed=None, exception=False):
        try:
            if displayed:
                WebDriverWait(self, timeout).until(EC.presence_of_element_located((By.ID, id_)) and
                                                   EC.visibility_of_element_located((By.ID, id_)))
            else:
                WebDriverWait(self, timeout).until(EC.presence_of_element_located((By.ID, id_)))
            element = self.find_element(by=By.ID, value=id_)
            element.jclick = lambda: self.element_click(element)
            element.aclick = lambda: self.element_action_click(element)
            return element
        except Exception as e:
            if exception:
                raise e

    def wait_css_element(self, cls, timeout=10, displayed=None, exception=False):
        try:
            if displayed:
                WebDriverWait(self, timeout).until(EC.presence_of_element_located((By.CSS_SELECTOR, cls)) and
                                                   EC.visibility_of_element_located((By.CSS_SELECTOR, cls)))
            else:
                WebDriverWait(self, timeout).until(EC.presence_of_element_located((By.CSS_SELECTOR, cls)))
            return self.find_element(by=By.CSS_SELECTOR, value=cls)
        except Exception as e:
            if exception:
                raise e

    def wait_xpath_element(self, path, timeout=10, delay=0, next_delay=0, displayed=None, exception=False):
        try:
            time.sleep(delay)
            if displayed:
                WebDriverWait(self, timeout).until(EC.presence_of_element_located((By.XPATH, path)) and
                                                   EC.visibility_of_element_located((By.XPATH, path)))
            else:
                WebDriverWait(self, timeout).until(EC.presence_of_element_located((By.XPATH, path)))
            element = self.find_element(by=By.XPATH, value=path)
            element.jclick = lambda: self.element_click(element)
            element.alick = lambda: self.element_action_click(element)
            time.sleep(next_delay)
            return element
        except Exception as e:
            if exception:
                raise e

    def wait_xpath_elements(self, path, timeout=10, delay=0, next_delay=0, displayed=None, exception=False):
        try:
            time.sleep(delay)
            if displayed:
                WebDriverWait(self, timeout).until(EC.presence_of_element_located((By.XPATH, path)) and
                                                   EC.visibility_of_element_located((By.XPATH, path)))
            else:
                WebDriverWait(self, timeout).until(EC.presence_of_element_located((By.XPATH, path)))
            elements = self.find_elements(by=By.XPATH, value=path)
            for e in elements:
                e.jclick = lambda: self.element_click(e)
                e.aclick = lambda: self.element_action_click(e)
            time.sleep(next_delay)
            return elements
        except Exception as e:
            if exception:
                raise e

    def regex_findall(self, rule, exception=False):
        try:
            text = self.page_source
            result = re.findall(rule, text)
            if len(result) == 0:
                raise Exception("not find")
            return result
        except Exception as e:
            if exception:
                raise e

    def wait_url(self, url, timeout=10, exception=False):
        try:
            while timeout > 0:
                time.sleep(1)
                timeout -= 1
                if str(self.current_url).startswith(url):
                    return

            raise Exception("TimeOut wait url")
        except Exception as e:
            if exception:
                raise e

    def save_screen(self, name, screenshot_path="./screenshot", delay=1):
        print(name)
        time.sleep(delay)
        os.makedirs(screenshot_path, exist_ok=True)
        path = os.path.join(screenshot_path, date_to_str(format_str="%Y-%m-%d %H:%M:%S ") + name + ".png")
        self.save_screenshot(path)
        self.screenshot_list.append(os.path.abspath(path))

    def wait_for_one(self, elements, timeout=10, check_second=0.5, exception=False):
        """
        r, v = self.firefox.wait_for_one({
            False: [{"re": "没有找到相关企业",displayed:False}, {"re": "我们只是确认一下你不是机器人"}],
            True: [{"css": ".tips-num"}]
        })

        :param elements:
        :param timeout:
        :param check_second:
        :return:
        """

        def find_by(by: str, value, displayed=None):
            find = False
            r_value = None
            if by == "xpath":
                try:
                    r_value = self.find_element(by=By.XPATH, value=value)
                    find = True
                except Exception:
                    pass
            elif by == "css":
                try:
                    r_value = self.find_element(by=By.CSS_SELECTOR, value=value)
                    find = True
                except Exception:
                    pass
            elif by == "id":
                try:
                    r_value = self.find_element(by=By.ID, value=value)
                    find = True
                except Exception:
                    pass
            elif by == "tag":
                try:
                    r_value = self.find_element(by=By.TAG_NAME, value=value)
                    find = True
                except Exception:
                    pass
            elif by == "re":
                text = self.page_source
                r_value = re.search(value, text)
                if r_value is not None:
                    find = True

            # 决断是否显示
            if displayed and r_value and r_value.is_displayed():
                find = True
            else:
                find = False

            return find, r_value

        while timeout > 0:
            for key in elements.keys():
                try:
                    el = elements[key]
                    if isinstance(el, list):
                        for e in el:
                            by, v, displayed = list(e.keys())[0], list(e.values())[0], e.get("displayed", None)
                            find, r_value = find_by(by, v, displayed)
                            if find:
                                return key, r_value
                    else:
                        by, v, displayed = list(el.keys())[0], list(el.values())[0], el.get("displayed", None)
                        find, r_value = find_by(by, v, displayed)
                        if find:
                            return key, r_value
                except Exception as e:
                    logging.exception(e)

            time.sleep(check_second)
            timeout -= check_second
        if exception:
            raise TimeoutError()

    def get_logs(self):
        logs = [json.loads(log['message'])['message'] for log in self.get_log('performance')]
        return logs

    def get_page_text_by_chrome(self, url=None, timeout=0, delay=0, scroll_end=False):
        """使用Chrome来获取页面源代码"""
        if timeout > 0:
            self.set_page_load_timeout(timeout)
            self.set_script_timeout(timeout)
        try:
            if url:
                self.open(url=url)
        except TimeoutException:
            pass
        time.sleep(delay)
        # 滚动到页面底
        if scroll_end:
            self.scroll_end_page()
        return self.page_source

    def js_loaction(self, url, timeout):
        try:
            self.execute_script(f"window.location.href='{url}'")
            time.sleep(1)
        except:
            pass
        self.wait_url(url, timeout)



import os
import time
import paramiko


def sh(cmd, host, username='root', password='', port=22):
    # print("ssh", host, "==>", cmd)
    ssh = paramiko.SSHClient()
    ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    ssh.connect(hostname=host, port=port, username=username, password=password)
    stdin, stdout, stderr = ssh.exec_command(cmd)
    res, err = stdout.read(), stderr.read()
    result = res if res else err
    # print("<==", result.decode())
    time.sleep(0.5)
    ssh.close()
    return result.decode()


def command(cmd, host, username='root', password='', nohup=False):
    ssh = paramiko.SSHClient()
    ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    ssh.connect(hostname=host, port=22, username=username,
                password=password)
    if nohup:
        cmd += ' & \n '
        invoke = ssh.invoke_shell()
        invoke.send(cmd)
        # 等待命令执行完成
        time.sleep(2)
    else:
        stdin, stdout, stderr = ssh.exec_command(cmd)
        result = stdout.read()
        return result
    ssh.close()


def scp_put(local_file, remote_file, host, username='root', password=''):
    sh(cmd=f"mkdir -p {os.path.dirname(remote_file)}", host=host, username=username, password=password)
    transport = paramiko.Transport((host, 22))
    transport.connect(username=username, password=password)
    sftp = paramiko.SFTPClient.from_transport(transport)
    sftp.put(local_file, remote_file)
    transport.close()


def scp_get(local_file, remote_file, host, username='root', password=''):
    transport = paramiko.Transport((host, 22))
    transport.connect(username=username, password=password)
    sftp = paramiko.SFTPClient.from_transport(transport)
    sftp.get(remote_file, local_file)
    transport.close()



# -*- coding: utf-8 -*-
import functools
import logging
import threading


def Async(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        my_thread = threading.Thread(target=func, args=args, kwargs=kwargs)
        my_thread.start()

    return wrapper


def Except(func):
    """捕获异常装饰器"""

    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            # serror_info = traceback.format_exc()
            # log.error(serror_info)
            # msg_box = QMessageBox(QMessageBox.Warning, '报错', serror_info)
            # msg_box.show()
            # msg_box.exec_()
            logging.exception(e)

    return wrapper



# -*- coding: utf-8 -*-
import re


class DictUtil(object):
    """
    将list数据（下划线式）转换成驼峰式的字典结构
    """

    def __init__(self, class_name, trans_type='camel'):
        func = self.to_lower_camel
        if trans_type == 'snake':
            func = self._to_snake
        for key, index in class_name.__dict__.items():
            if '__' in key:
                continue
            new_key = func(key)
            setattr(self, new_key, index)

    @staticmethod
    def to_lower_camel(name: str):
        """下划线转小驼峰法命名"""
        return re.sub('_([a-zA-Z])', lambda m: (m.group(1).upper()), name.lower())

    @staticmethod
    def to_uper_camel(name: str):
        """下划线转大写驼峰法命名"""
        a = re.sub('_([a-zA-Z])', lambda m: (m.group(1).upper()), name.lower())
        return a[0].upper() + a[1:]

    def _to_snake(self, name: str) -> str:
        """驼峰转下划线"""
        if '_' not in name:
            name = re.sub(r'([a-z])([A-Z])', r'\1_\2', name)
        else:
            raise ValueError(f'{name}字符中包含下划线，无法转换')
        return name.lower()

    def list2dict(self, arrays):
        return {key: arrays[index] for key, index in self.__dict__.items()}

    @staticmethod
    def dict_arrays_keyupper(arrays):
        arrays_new = []
        for obj in arrays:
            arrays_new.append({DictUtil.to_uper_camel(key): obj[key] for key in obj.keys()})
        return arrays_new

    @staticmethod
    def dict_keyupper(dict_obj):
        return {str(key).upper(): value for key, value in dict_obj}


if __name__ == '__main__':
    print(DictUtil.to_lower_camel("qqq_cvvv"))



import os
import random


def get_random_user_agent():
    pwd = os.path.dirname(__file__)
    ua_list = open(f"{pwd}/pcua.txt").readlines()
    u = random.choice(ua_list).replace("\n", "")
    while u.strip() == "":
        u = random.choice(ua_list).replace("\n", "")
    return u.strip()


def get_firefox_user_agent():
    pwd = os.path.dirname(__file__)
    ua_list = open(f"{pwd}/pcua.txt").readlines()
    u = random.choice(ua_list).replace("\n", "")
    while u.strip() == "" and "Firefox" not in u:
        u = random.choice(ua_list).replace("\n", "")
    return u.strip()


if __name__ == '__main__':
    print(get_random_user_agent())



import os

from selenium.webdriver.chrome.options import Options
from selenium.webdriver.chrome.webdriver import WebDriver
from webdriver_manager.chrome import ChromeDriverManager

from db_hammer.auto.base_driver import BaseDriver


class ChromeDriver(BaseDriver, WebDriver):

    def __init__(self, headless=True,
                 rec_log=False,
                 user_data_dir=None,
                 default_directory=None,
                 disable_images=False,
                 auto_tip_close=False,
                 no_sandbox=True,
                 proxy=None):
        self.udata = {}
        self.screenshot_list = []
        caps = {
            'browserName': 'chrome',
            'loggingPrefs': {
                'browser': 'ALL',
                'driver': 'ALL',
                'performance': 'ALL',
            },
            'goog:chromeOptions': {
                'perfLoggingPrefs': {
                    'enableNetwork': True,
                },
                'w3c': False,
            },
        }
        prefs = {}

        if os.path.exists(".webdriver"):
            executable_path = open(".webdriver", "r").read()
        else:
            executable_path = ChromeDriverManager().install()
            open(".webdriver", "w+").write(executable_path)

        # d = DesiredCapabilities.CHROME
        # d['loggingPrefs'] = {'performance': 'ALL'}

        chrome_options = Options()
        if auto_tip_close:
            chrome_options.add_experimental_option("excludeSwitches", ["enable-automation"])
            chrome_options.add_experimental_option("useAutomationExtension", 'False')
        chrome_options.accept_insecure_certs = True
        if user_data_dir:
            chrome_options.add_argument(f"user-data-dir={user_data_dir}")
        if headless:
            chrome_options.add_argument("--headless")
        if no_sandbox:
            chrome_options.add_argument('--disable-gpu')
            chrome_options.add_argument('--no-sandbox')
            chrome_options.add_argument("--disable-extensions")
            chrome_options.add_argument('--hide-scrollbars')
        if rec_log:
            chrome_options.add_experimental_option('w3c', False)
        if proxy:
            chrome_options.add_argument(f"--proxy-server={proxy}")
        else:
            chrome_options.add_argument('--no-proxy-server')
        if default_directory:
            os.makedirs(default_directory, exist_ok=True)

            prefs.update({"profile.default_content_settings.popups": 0,
                          "download.default_directory": default_directory,
                          'safebrowsing.enabled': False,
                          'safebrowsing.disable_download_protection': True,
                          'download.prompt_for_download': False,
                          "directory_upgrade": True})
        if disable_images:
            prefs.update({"profile.managed_default_content_settings.images": 2})

        chrome_options.add_experimental_option('prefs', prefs)
        super().__init__(executable_path=executable_path, options=chrome_options)
        # super().__init__(executable_path=executable_path, options=chrome_options)
        self.execute_cdp_cmd("Page.addScriptToEvaluateOnNewDocument", {
            "source": """
            Object.defineProperty(navigator, 'webdriver', {
              get: () => undefined
            })
          """
        })
        self.execute_cdp_cmd("Emulation.setUserAgentOverride", {
            "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.77 Safari/537.36"
            # "userAgent": "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 micromessenger/8.0.15(0x18000f27) NetType/4G Language/zh_CN"
        })
        self.set_window_position(0, 0)
        self.set_window_size(1200, 1000)



def cut_list_data(data_list: list, cut_num=1) -> list:
    """
    把数组切成指定份数返回数据数组
    :param data_list: 源数组
    :param cut_num: 切份数
    :return:
    """
    # 数据的份数
    data_nums = len(data_list)
    # 数据分片开始
    cut_start = 0
    # 分片的数量
    # 分片大小,为0表示不分片
    cut_size = data_nums // cut_num
    yu = data_nums % cut_num
    results = []
    for i in range(0, cut_num):
        # 先分余
        if yu > 0:
            yu -= 1
            cut_end = cut_start + cut_size + 1
        else:
            cut_end = cut_start + cut_size
        # 是不是最后的一片
        if cut_end > data_nums - cut_size:
            cut_end = data_nums
        results.append(data_list[cut_start:cut_end])
        cut_start = cut_end
    return results


def cut_list_index(data_list: list, cut_num=1):
    """
    把数组切成指定份数返回数据下标数组
    :param data_list: 源数组
    :param cut_num: 切份数
    :return:
    """
    if data_list is None:
        return None
    # 数据的份数
    data_nums = len(data_list)
    # 数据分片开始
    cut_start = 0
    # 分片的数量
    # 分片大小,为0表示不分片
    cut_size = data_nums // cut_num
    yu = data_nums % cut_num
    results = []
    for i in range(0, cut_num):
        # 先分余
        if yu > 0:
            yu -= 1
            cut_end = cut_start + cut_size + 1
        else:
            cut_end = cut_start + cut_size
        # 是不是最后的一片
        if cut_end > data_nums - cut_size:
            cut_end = data_nums
        results.append((cut_start, cut_end))
        cut_start = cut_end
    return results


def cut_list_size(data_list: list, cut_size=1):
    """
    把数组按指定大小切数组
    :param data_list: 源数组
    :param cut_size: 按指定size切
    :return:
    """
    if data_list is None:
        return None
    # 数据的份数
    data_nums = len(data_list)
    tc = data_nums // cut_size
    yu = data_nums % cut_size
    results = []
    c = 0
    for i in range(0, tc):
        if (i + 1) * cut_size <= data_nums:
            results.append(data_list[i * cut_size:(i + 1) * cut_size])
    if yu > 0:
        results.append(data_list[tc * cut_size:])
    return results


def cut_list_size_index(data_list: list, cut_size=1):
    """
    把数组按指定大小切数组下标
    :param data_list: 源数组
    :param cut_size: 按指定size切
    :return:
    """
    if data_list is None:
        return None
    # 数据的份数
    data_nums = len(data_list)
    tc = data_nums // cut_size
    yu = data_nums % cut_size
    results = []
    c = 0
    for i in range(0, tc):
        if (i + 1) * cut_size <= data_nums:
            results.append([i * cut_size, (i + 1) * cut_size])
    if yu > 0:
        results.append([tc * cut_size, data_nums])
    return results


if __name__ == '__main__':
    data = [1, 2, 3, 4, 5, 6, 7, 8, 9]

    print("把数组切成指定份数返回数据数组")
    for i in range(1, 10):
        print(f"------------切成{i}份------------")
        for r in cut_list_data(data, i):
            print(r)

    print("把数组切成指定份数返回数据下标数组")
    for i in range(1, 10):
        print(f"------------切成{i}份------------")
        for r in cut_list_index(data, i):
            s, e = r
            print(s, e, "---->", data[s:e])

    print("把数组按指定大小切数组")
    for i in range(1, 10):
        print(f"------------切成{i}大小------------")
        for r in cut_list_size(data, i):
            print(r)

    print("把数组按指定大小切数组下标")
    for i in range(1, 10):
        print(f"------------切成{i}大小下标------------")
        for r in cut_list_size_index(data, i):
            s, e = r
            print(s, e, "---->", data[s:e])



import os

from selenium.webdriver.firefox import webdriver
from selenium.webdriver.firefox.webdriver import WebDriver
from webdriver_manager.firefox import GeckoDriverManager
from db_hammer.auto.base_driver import BaseDriver


class FirefoxDriver(BaseDriver, WebDriver):

    def __init__(self, headless=True, user_data_dir=None, default_directory=None, disable_images=False,
                 auto_tip_close=False, proxy=None, user_agent=None, fast=True):
        self.udata = {}
        if user_data_dir:
            profile = webdriver.FirefoxProfile(user_data_dir)
        else:
            profile = webdriver.FirefoxProfile()
        if proxy:
            proxy_ip, proxy_port = proxy.split(":")
            proxy_ip = proxy_ip.replace("http://", "")
            proxy_port = int(proxy_port)
            profile.set_preference('network.proxy.type', 1)
            profile.set_preference('network.proxy.http', proxy_ip)
            profile.set_preference('network.proxy.http_port', proxy_port)
            profile.set_preference('network.proxy.ssl', proxy_ip)
            profile.set_preference('network.proxy.ssl_port', proxy_port)
        if default_directory:
            profile.set_preference("browser.download.folderList", 2)
            profile.set_preference("browser.download.dir", default_directory)
            profile.set_preference("browser.helperApps.neverAsk.saveToDisk", "binary/octet-stream")

        profile.set_preference("browser.privatebrowsing.autostart", True)
        if disable_images:
            profile.set_preference('permissions.default.stylesheet', 2)
            # Disable images
            profile.set_preference('permissions.default.image', 2)
            # profile.set_preference("network.http.use-cache", False)
            # profile.set_preference("browser.cache.memory.enable", False)
            profile.set_preference("browser.cache.disk.enable", False)
            profile.set_preference("browser.sessionhistory.max_total_viewers", 3)
            # profile.set_preference("network.dns.disableIPv6", True)
            profile.set_preference("Content.notify.interval", 750000)
            profile.set_preference("content.notify.backoffcount", 3)
        options = webdriver.Options()
        options.add_argument("--ignore-certificate-errors")
        if auto_tip_close:
            options.add_argument("--disable-infobars")
        # options.set_preference("dom.webdriver.enabled", False)
        if headless:
            options.add_argument('-headless')
            options.add_argument('--disable-gpu')
        if fast:
            profile.set_preference("network.http.proxy.pipelining", True)
            profile.set_preference("network.http.pipelining.maxrequests", 8)
            profile.set_preference("content.notify.interval", 500000)
            profile.set_preference("content.notify.ontimer", True)
            profile.set_preference("content.switch.threshold", 250000)
            profile.set_preference("browser.cache.memory.capacity", 65536)  # Increase the cache capacity.
            profile.set_preference("browser.startup.homepage", "about:blank")
            profile.set_preference("reader.parse-on-load.enabled", False)  # Disable reader, we won't need that.
            profile.set_preference("browser.pocket.enabled", False)  # Duck pocket too!
            profile.set_preference("loop.enabled", False)
            profile.set_preference("browser.chrome.toolbar_style", 1)  # Text on Toolbar instead of icons
            profile.set_preference("browser.display.show_image_placeholders",
                                   False)  # Don't show thumbnails on not loaded images.
            profile.set_preference("browser.display.use_document_colors", False)  # Don't show document colors.
            profile.set_preference("browser.display.use_document_fonts", 0)  # Don't load document fonts.
            profile.set_preference("browser.display.use_system_colors", True)  # Use system colors.
            profile.set_preference("browser.formfill.enable", False)  # Autofill on forms disabled.
            profile.set_preference("browser.helperApps.deleteTempFileOnExit", True)  # Delete temprorary files.
            profile.set_preference("browser.shell.checkDefaultBrowser", False)
            profile.set_preference("browser.startup.homepage", "about:blank")
            profile.set_preference("browser.startup.page", 0)  # blank
            profile.set_preference("browser.tabs.forceHide", True)  # Disable tabs, We won't need that.
            profile.set_preference("browser.urlbar.autoFill", False)  # Disable autofill on URL bar.
            profile.set_preference("browser.urlbar.autocomplete.enabled", False)  # Disable autocomplete on URL bar.
            profile.set_preference("browser.urlbar.showPopup", False)  # Disable list of URLs when typing on URL bar.
            profile.set_preference("browser.urlbar.showSearch", False)  # Disable search bar.
            profile.set_preference("extensions.checkCompatibility", False)  # Addon update disabled
            profile.set_preference("extensions.checkUpdateSecurity", False)
            profile.set_preference("extensions.update.autoUpdateEnabled", False)
            profile.set_preference("extensions.update.enabled", False)
            profile.set_preference("general.startup.browser", False)
            profile.set_preference("plugin.default_plugin_disabled", False)
            profile.set_preference("permissions.default.image", 2)  # Image load disabled again

        if user_agent:
            if user_agent == "random":
                user_agent = get_random_user_agent()
            options.add_argument('user-agent=' + user_agent)

        if os.path.exists(".firefoxdriver"):
            executable_path = open(".firefoxdriver", "r").read()
        else:
            executable_path = GeckoDriverManager(cache_valid_range=20).install()
            open(".firefoxdriver", "w+").write(executable_path)

        super().__init__(executable_path=executable_path,
                         options=options,
                         firefox_profile=profile)
